/**
    FREEZER - A tray icon for Deezer
    
    DeSerializer.java originally from Solenopsis Framework (C) 2006 Amos Brocco

    Copyright (C) 2007 Amos Brocco <amos.brocco@unifr.ch>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class DeSerializer {

	private InputStreamReader streamReader;

	private Object dataTree;

	private char ch;

	private boolean skipNextRead = false;

	public DeSerializer(InputStream serialized) throws IOException {
		streamReader = new InputStreamReader(serialized);
		dataTree = readValue();
		serialized.close();
		streamReader.close();
	}

	private char read() throws IOException {
		if (!skipNextRead)
			ch = (char) streamReader.read();
		else
			skipNextRead = false;
		return ch;
	}

	private void unread() {
		skipNextRead = true;
	}

	private Object readValue() throws IOException {
		read();
		if (Character.isDigit(ch)) {
			return readString();
		} else if (ch == 'd') {
			return readDictionary();
		} else if (ch == 'l') {
			return readList();
		} else {
			return null;
		}
	}

	@SuppressWarnings("unchecked")
	private List readList() throws IOException {
		ArrayList<Object> list = new ArrayList<Object>();
		read();
		if (ch == 'e')
			return list;
		else
			unread();
		while (true) {
			Object item = readValue();
			if (item == null)
				break;
			list.add(item);
			read();
			unread();
		}
		return list;
	}

	private String readString() throws IOException {
		String slen = new String();
		while (Character.isDigit(ch)) {
			slen = slen + ch;
			read();
		}
		int len = new Integer(slen).intValue();
		String str = new String();
		for (int i = 0; i < len; i++) {
			str += read();
		}
		return str;
	}

	private Map<String, Object> readDictionary() throws IOException {
		HashMap<String, Object> dict = new HashMap<String, Object>();
		while (true) {
			Object key = readValue();
			if ((key == null) || !(key instanceof String))
				break;
			Object value = readValue();
			dict.put((String) key, value);
		}
		return dict;
	}

	public Object get() {
		return dataTree;
	}

}
